import { __awaiter, __decorate } from "tslib";
// eslint-disable-next-line import/no-extraneous-dependencies
import { Connect, Datasource } from '@genesislcap/foundation-comms';
import { FASTElement, customElement, attr, observable } from '@microsoft/fast-element';
import { logger } from '../utils';
import { SERVICE_STATUS_LOGGER_PREFIX as LOGGER_PREFIX, SERVICE_STATUS_EVENTS_NAMES as EVENTS_NAMES, SERVICE_STATUS_CLOSE_TIMEOUT as CLOSE_TIMEOUT, SERVICE_STATUS_EVENTS_NAMES, } from './main.config';
import { foundationServicesStatusStyles as styles } from './main.styles';
import { getPrefixedServicesStatus as getTemplate } from './main.template';
/**
 * The `ServicesStatus` component is designed to monitor and display the status of various services in real-time.
 * It leverages a connection to a backend datasource to receive updates about services status
 * and displays these updates to the user.
 *
 * @public
 */
export class ServicesStatus extends FASTElement {
    constructor() {
        super(...arguments);
        this.allMessages = [];
        this.displayMessages = [];
        this.resourceName = 'ALL_PROCESSES_STATUS';
        this.closeTimeout = CLOSE_TIMEOUT;
        this.autoClose = false;
        this.maxMessages = 1;
        this.messageRestored = `Thank you for your patience while we were restoring service. The [SERVICE_NAME] service is now available as normal. Please refresh your page to update`;
        this.messageFailed = `We're sorry, [SERVICE_NAME] service is experiencing technical difficulties. It is recommended that you save your data before disconnecting from the application. Data is not current and may be incomplete`;
        this.titleRestored = `[SERVICE_NAME] Service back online`;
        this.titleFailed = `[SERVICE_NAME] Service outage`;
        this.eventNameCloseMessage = EVENTS_NAMES.CLOSE_MESSAGE;
        this.eventNameAutoCloseMessage = EVENTS_NAMES.AUTO_CLOSE_MESSAGE;
        this.closeMessageHandler = (event) => {
            const serviceId = event.detail;
            this.allMessages = this.allMessages.filter((message) => message.id !== serviceId);
        };
        this.getTextHelper = (textType, type, item) => {
            const getterByType = {
                title: {
                    restored: this.titleRestored,
                    failed: this.titleFailed,
                },
                message: {
                    restored: this.messageRestored,
                    failed: this.messageFailed,
                },
            };
            const message = getterByType[textType][type];
            return this.replaceTextPlaceholders(message, item);
        };
    }
    allMessagesChanged() {
        this.$emit(SERVICE_STATUS_EVENTS_NAMES.SERVICES_CHANGED, this.allMessages);
        logger.debug(`${LOGGER_PREFIX} Services changed`, this.allMessages);
        this.updateDisplayMessages();
    }
    maxMessagesChanged() {
        this.updateDisplayMessages();
    }
    connectedCallback() {
        super.connectedCallback();
        this.startMonitoring();
    }
    disconnectedCallback() {
        this.stopMonitoring();
    }
    addMessage(newMessage) {
        if (Array.isArray(newMessage)) {
            this.allMessages = [...this.allMessages, ...newMessage];
        }
        else {
            this.allMessages = [...this.allMessages, newMessage];
        }
        this.updateDisplayMessages();
    }
    updateDisplayMessages() {
        this.displayMessages = this.maxMessages
            ? this.allMessages.slice(-this.maxMessages)
            : this.allMessages;
    }
    prepareServiceItem(serverItem) {
        return {
            id: serverItem.DETAILS.ROW_REF,
            isWorking: serverItem.PROCESS_STATUS,
            name: serverItem.PROCESS_NAME,
        };
    }
    startMonitoring() {
        return __awaiter(this, void 0, void 0, function* () {
            const datasourceInit = yield this.datasource.init({
                resourceName: this.resourceName,
            }, true);
            if (!datasourceInit) {
                logger.debug(`${LOGGER_PREFIX} Failed to initialise datasource`);
                return;
            }
            const { ROW: snapshotRows } = yield this.datasource.snapshot();
            this.datasourceSubscription = this.datasource.stream.subscribe({
                next: (result) => this.handleDataStream(result, snapshotRows),
                error: (err) => logger.error(`${LOGGER_PREFIX} Stream error:`, err),
            });
        });
    }
    handleDataStream(result, snapshotRows) {
        const { ROW: rows } = result;
        const newMessages = [];
        rows === null || rows === void 0 ? void 0 : rows.forEach((item) => {
            const changedRowRef = item.DETAILS.ROW_REF;
            if ('PROCESS_STATUS' in item && result.SEQUENCE_ID !== 1) {
                const itemSnapshot = snapshotRows.find(({ DETAILS: { ROW_REF } }) => ROW_REF === changedRowRef);
                if (itemSnapshot) {
                    const newServiceItem = this.prepareServiceItem(Object.assign(Object.assign({}, itemSnapshot), item));
                    const targetEvent = newServiceItem.isWorking
                        ? EVENTS_NAMES.SERVICE_RESTORED
                        : EVENTS_NAMES.SERVICE_FAIL;
                    newMessages.push(newServiceItem);
                    this.$emit(targetEvent, newServiceItem);
                }
            }
        });
        if (newMessages.length) {
            this.addMessage(newMessages);
        }
    }
    stopMonitoring() {
        var _a;
        (_a = this.datasourceSubscription) === null || _a === void 0 ? void 0 : _a.unsubscribe();
    }
    emitCloseMessage(target, identifier) {
        target.dispatchEvent(new CustomEvent(this.eventNameCloseMessage, {
            bubbles: true,
            composed: true,
            detail: identifier,
        }));
    }
    replaceTextPlaceholders(template, item) {
        var _a, _b, _c;
        return template
            .replace(/\[SERVICE_NAME\]/g, (_a = item.name) !== null && _a !== void 0 ? _a : 'Unknown Name')
            .replace(/\[SERVICE_ID\]/g, (_c = (_b = item.id) === null || _b === void 0 ? void 0 : _b.toString()) !== null && _c !== void 0 ? _c : 'Unknown ID');
    }
}
__decorate([
    Connect
], ServicesStatus.prototype, "connect", void 0);
__decorate([
    Datasource
], ServicesStatus.prototype, "datasource", void 0);
__decorate([
    observable
], ServicesStatus.prototype, "allMessages", void 0);
__decorate([
    observable
], ServicesStatus.prototype, "displayMessages", void 0);
__decorate([
    attr({ attribute: 'resource-name' })
], ServicesStatus.prototype, "resourceName", void 0);
__decorate([
    attr({ attribute: 'close-timeout' })
], ServicesStatus.prototype, "closeTimeout", void 0);
__decorate([
    attr({ attribute: 'auto-close', mode: 'boolean' })
], ServicesStatus.prototype, "autoClose", void 0);
__decorate([
    attr({ attribute: 'max-messages' })
], ServicesStatus.prototype, "maxMessages", void 0);
__decorate([
    attr({ attribute: 'message-restored' })
], ServicesStatus.prototype, "messageRestored", void 0);
__decorate([
    attr({ attribute: 'message-failed' })
], ServicesStatus.prototype, "messageFailed", void 0);
__decorate([
    attr({ attribute: 'title-restored' })
], ServicesStatus.prototype, "titleRestored", void 0);
__decorate([
    attr({ attribute: 'title-failed' })
], ServicesStatus.prototype, "titleFailed", void 0);
/**
 * ZeroServicesStatus.
 *
 * @remarks
 * A zero version that pre-registers zero components and uses the zero design system.
 *
 * @example
 * ```ts
 * import { ZeroServicesStatus } from '@genesislcap/foundation-services-status';
 * ...
 * ZeroServicesStatus
 * ```
 *
 * @example Load the micro frontend on-demand
 * ```ts

 *   const { ZeroServicesStatus } = await import('@genesislcap/foundation-services-status');
 *   return ZeroServicesStatus;
 * },
 * ```
 *
 * @public
 */
let ZeroServicesStatus = class ZeroServicesStatus extends ServicesStatus {
    /**
     * @internal
     */
    connectedCallback() {
        const _super = Object.create(null, {
            connectedCallback: { get: () => super.connectedCallback }
        });
        return __awaiter(this, void 0, void 0, function* () {
            _super.connectedCallback.call(this);
            yield this.loadRemotes();
        });
    }
    /**
     * @internal
     */
    loadRemotes() {
        return __awaiter(this, void 0, void 0, function* () {
            const { registerCommonZeroComponents } = yield import('../components/zero-components');
            yield registerCommonZeroComponents();
        });
    }
};
ZeroServicesStatus = __decorate([
    customElement({
        name: 'zero-services-status',
        template: getTemplate('zero'),
        styles,
    })
], ZeroServicesStatus);
export { ZeroServicesStatus };
